@@ -1,6 +1,6 @@
 /*
     common
-    copyright (c) 1998-2017 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2017 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -233,7 +233,7 @@ MemoryCopy (LPVOID  lpvDest,
     lpvBuf1,バッファ2
      uBytes,バイト数
         RET,0:等しい,負:バッファ1は小さい,正:バッファ1は大きい              */
-INT WINAPI
+int WINAPI
 MemoryCompare (LPCVOID lpvBuf1,
                LPCVOID lpvBuf2,
                SIZE_T  uBytes)
@@ -323,24 +323,681 @@ MemoryDuplicate (LPCVOID lpMemory,
 }
 
 
-/*  文字列を確保する
-    lpszText,文字列
-         RET,確保したメモリ,NULL:エラー                                     */
-LPWSTR WINAPI
-MemoryStringW (LPCWSTR lpszText)
+/******************************************************************************
+*                                                                             *
+******************************************************************************/
+/*  文字列を書式化する
+    ppszString,文字列
+    lpszFormat,書式文字列
+       arglist,引数
+           RET,文字数(終端のNULLを含まない)                                 */
+SSIZE_T WINAPI
+wvasprintfW (LPWSTR  *ppszString,
+             LPCWSTR  lpszFormat,
+             va_list  arglist)
 {
-  return MemoryDuplicate (lpszText,
-                                (lstrlenW (lpszText) + 1) * sizeof (WCHAR));
+  SSIZE_T nResult = -1;
+
+  if (lpszFormat && arglist)
+    {
+      LPWSTR lpszText = NULL;
+      SSIZE_T nSize = 0;
+
+      do
+        {
+          nSize += 4096;
+          lpszText = MemoryReAlloc (lpszText, nSize * sizeof (WCHAR));
+          nResult = wvnsprintfW (lpszText, nSize, lpszFormat, arglist);
+        }
+      while (nResult < 0 || nResult >= nSize - 1);
+      if (ppszString)
+        *ppszString = MemoryReAlloc (lpszText, (nResult + 1) * sizeof (WCHAR));
+      else
+        MemoryFree (lpszText);
+    }
+  return nResult;
 }
 
 
-/*  文字列を確保する
-    lpszText,文字列
-         RET,確保したメモリ,NULL:エラー                                     */
-LPSTR WINAPI
-MemoryStringA (LPCSTR lpszText)
+/*  文字列を書式化する
+    ppszString,文字列
+    lpszFormat,書式文字列
+       arglist,引数
+           RET,文字数(終端のNULLを含まない)                                 */
+SSIZE_T WINAPI
+wvasprintfA (LPSTR  *ppszString,
+             LPCSTR  lpszFormat,
+             va_list arglist)
 {
-  return MemoryDuplicate (lpszText, (lstrlenA (lpszText) + 1) * sizeof (CHAR));
+  SSIZE_T nResult = -1;
+
+  if (lpszFormat && arglist)
+    {
+      LPSTR lpszText = NULL;
+      SSIZE_T nSize = 0;
+
+      do
+        {
+          nSize += 4096;
+          lpszText = MemoryReAlloc (lpszText, nSize * sizeof (CHAR));
+          nResult = wvnsprintfA (lpszText, nSize, lpszFormat, arglist);
+        }
+      while (nResult < 0 || nResult >= nSize - 1);
+      if (ppszString)
+        *ppszString = MemoryReAlloc (lpszText, (nResult + 1) * sizeof (CHAR));
+      else
+        MemoryFree (lpszText);
+    }
+  return nResult;
+}
+
+
+/*  文字列を書式化する
+    ppszString,文字列
+      lpFormat,書式文字列
+           RET,文字数(終端のNULLを含まない)                                 */
+SSIZE_T WINAPI
+wasprintfW (LPWSTR  *ppszString,
+            LPCWSTR  lpszFormat, ...)
+{
+  int nResult;
+  va_list arglist;
+
+  va_start (arglist, lpszFormat);
+  nResult = wvasprintfW (ppszString, lpszFormat, arglist);
+  va_end (arglist);
+  return nResult;
+}
+
+
+/*  文字列を書式化する
+    ppszString,文字列
+      lpFormat,書式文字列
+           RET,文字数(終端のNULLを含まない)                                 */
+SSIZE_T WINAPI
+wasprintfA (LPSTR  *ppszString,
+            LPCSTR  lpszFormat, ...)
+{
+  int nResult;
+  va_list arglist;
+
+  va_start (arglist, lpszFormat);
+  nResult = wvasprintfA (ppszString, lpszFormat, arglist);
+  va_end (arglist);
+  return nResult;
+}
+
+
+/*  文字列を複製する
+    lpszText,文字列
+       nSize,文字数(負:NULL終端文字列)
+         RET,複製したメモリ,NULL:エラー                                     */
+LPWSTR WINAPI
+StringDuplicateExW (LPCWSTR lpszText,
+                    SSIZE_T nSize)
+{
+  LPWSTR lpszResult;
+
+  if (nSize > 0)
+    {
+      if (lpszText && lpszText[nSize - 1] != '\0')
+        nSize++;
+    }
+  else if (lpszText && nSize < 0)
+    {
+      nSize = lstrlenW (lpszText) + 1;
+    }
+  if (nSize > 0)
+    {
+      lpszResult = MemoryAlloc (nSize * sizeof (WCHAR));
+      MemoryCopy (lpszResult, lpszText, (nSize - 1) * sizeof (WCHAR));
+    }
+  else
+    {
+      lpszResult = NULL;
+    }
+  return lpszResult;
+}
+
+
+/*  文字列を複製する
+    lpszText,文字列
+       nSize,文字数(負:NULL終端文字列)
+         RET,複製したメモリ,NULL:エラー                                     */
+LPSTR WINAPI
+StringDuplicateExA (LPCSTR  lpszText,
+                    SSIZE_T nSize)
+{
+  LPSTR lpszResult;
+
+  if (nSize > 0)
+    {
+      if (lpszText && lpszText[nSize - 1] != '\0')
+        nSize++;
+    }
+  else if (lpszText && nSize < 0)
+    {
+      nSize = lstrlenA (lpszText) + 1;
+    }
+  if (nSize > 0)
+    {
+      lpszResult = MemoryAlloc (nSize * sizeof (CHAR));
+      MemoryCopy (lpszResult, lpszText, (nSize - 1) * sizeof (CHAR));
+    }
+  else
+    {
+      lpszResult = NULL;
+    }
+  return lpszResult;
+}
+
+
+/*  文字列を複製する
+    lpszText,文字列
+         RET,複製したメモリ,NULL:エラー                                     */
+LPWSTR WINAPI
+StringDuplicateW (LPCWSTR lpszText)
+{
+  return StringDuplicateExW (lpszText, -1);
+}
+
+
+/*  文字列を複製する
+    lpszText,文字列
+         RET,複製したメモリ,NULL:エラー                                     */
+LPSTR WINAPI
+StringDuplicateA (LPCSTR lpszText)
+{
+  return StringDuplicateExA (lpszText, -1);
+}
+
+
+/*  マルチバイト文字列をワイド文字列に変換する
+    lpszMultiByte,マルチバイト文字列
+     cchMultiByte,文字数(負:NULL終端文字列)
+              RET,ワイド文字列                                              */
+LPWSTR WINAPI
+StringMultiByteToWideCharEx (LPCSTR  lpszMultiByte,
+                             SSIZE_T cchMultiByte)
+{
+  LPWSTR lpszWideChar = NULL;
+
+  if (lpszMultiByte && cchMultiByte != 0)
+    {
+      int cchWideChar;
+
+      if (cchMultiByte < 0)
+        cchMultiByte = -1;
+      cchWideChar = MultiByteToWideChar (CP_ACP, 0,
+                                        lpszMultiByte, cchMultiByte, NULL, 0);
+      if (cchWideChar > 0)
+        {
+          lpszWideChar = MemoryAlloc (cchWideChar * sizeof (WCHAR));
+          if (MultiByteToWideChar (CP_ACP, 0, lpszMultiByte, cchMultiByte,
+                                            lpszWideChar, cchWideChar) <= 0)
+            {
+              MemoryFree (lpszWideChar);
+              lpszWideChar = NULL;
+            }
+          else if (lpszWideChar[cchWideChar - 1] != '\0')
+            {
+              lpszWideChar = MemoryReAlloc (lpszWideChar,
+                                        (cchWideChar + 1) * sizeof (WCHAR));
+            }
+        }
+    }
+  return lpszWideChar;
+}
+
+
+/*  ワイド文字列をマルチバイト文字列に変換する
+    lpszWideChar,ワイド文字列
+     cchWideChar,文字数(負:NULL終端文字列)
+             RET,マルチバイト文字列                                         */
+LPSTR WINAPI
+StringWideCharToMultiByteEx (LPCWSTR lpszWideChar,
+                             SSIZE_T cchWideChar)
+{
+  LPSTR lpszMultiByte = NULL;
+
+  if (lpszWideChar && cchWideChar != 0)
+    {
+      int cchMultiByte;
+
+      if (cchWideChar < 0)
+        cchWideChar = -1;
+      cchMultiByte = WideCharToMultiByte (CP_ACP, 0, lpszWideChar, cchWideChar,
+                                                        NULL, 0, NULL, NULL);
+      if (cchMultiByte > 0)
+        {
+          lpszMultiByte = MemoryAlloc (cchMultiByte * sizeof (CHAR));
+          if (WideCharToMultiByte (CP_ACP, 0, lpszWideChar, cchWideChar,
+                                lpszMultiByte, cchMultiByte, NULL, NULL) <= 0)
+            {
+              MemoryFree (lpszMultiByte);
+              lpszMultiByte = NULL;
+            }
+          else if (lpszMultiByte[cchMultiByte - 1] != '\0')
+            {
+              lpszMultiByte = MemoryReAlloc (lpszMultiByte,
+                                        (cchMultiByte + 1) * sizeof (CHAR));
+            }
+        }
+    }
+  return lpszMultiByte;
+}
+
+
+/*  マルチバイト文字列をワイド文字列に変換する
+    lpszMultiByte,マルチバイト文字列
+              RET,ワイド文字列                                              */
+LPWSTR WINAPI
+StringMultiByteToWideChar (LPCSTR lpszMultiByte)
+{
+  return StringMultiByteToWideCharEx (lpszMultiByte, -1);
+}
+
+
+/*  ワイド文字列をマルチバイト文字列に変換する
+    lpszWideChar,ワイド文字列
+             RET,マルチバイト文字列                                         */
+LPSTR WINAPI
+StringWideCharToMultiByte (LPCWSTR lpszWideChar)
+{
+  return StringWideCharToMultiByteEx (lpszWideChar, -1);
+}
+
+
+/*  文字列を正規化する
+    lpszString,文字列
+         nSize,文字数(負:NULL終端文字列)
+           RET,正規化された文字列                                           */
+LPWSTR WINAPI
+StringCanonicalizeExW (LPCWSTR lpszString,
+                       SSIZE_T nSize)
+{
+  LPWSTR lpszResult;
+
+  if (lpszString)
+    {
+      int nLen;
+      LPCWSTR p, r;
+      LPWSTR q;
+
+      nLen = nSize >= 0 ? nSize : lstrlenW (lpszString);
+      p = lpszString;
+      r = p + nLen;
+      q = lpszResult = MemoryAlloc ((nLen * 6 + 3) * sizeof (WCHAR));
+      *q++ = '\"';
+      while (p < r)
+        {
+          switch (*p)
+            {
+              case '\a': *q++ = '\\'; *q++ = 'a';  break;
+              case '\b': *q++ = '\\'; *q++ = 'b';  break;
+              case '\n': *q++ = '\\'; *q++ = 'n';  break;
+              case '\r': *q++ = '\\'; *q++ = 'r';  break;
+              case '\f': *q++ = '\\'; *q++ = 'f';  break;
+              case '\t': *q++ = '\\'; *q++ = 't';  break;
+              case '\v': *q++ = '\\'; *q++ = 'v';  break;
+              case '\\': *q++ = '\\'; *q++ = '\\'; break;
+              case '\?': *q++ = '\\'; *q++ = '\?'; break;
+              case '\'': *q++ = '\\'; *q++ = '\''; break;
+              case '\"': *q++ = '\\'; *q++ = '\"'; break;
+              default:
+                if (0x20 <= *p && *p <= 0x7e)
+                  {
+                    *q++ = *p;
+                  }
+                else
+                  {
+                    wsprintfW (q, L"\\x%04X", *p);
+                    q += 6;
+                  }
+            }
+          p++;
+        }
+      *q++ = '\"';
+      *q = '\0';
+    }
+  else
+    {
+      lpszResult = StringDuplicateW (L"NULL");
+    }
+  return lpszResult;
+}
+
+
+/*  文字列を正規化する
+    lpszString,文字列
+         nSize,文字数(負:NULL終端文字列)
+           RET,正規化された文字列                                           */
+LPSTR WINAPI
+StringCanonicalizeExA (LPCSTR  lpszString,
+                       SSIZE_T nSize)
+{
+  LPSTR lpszResult;
+
+  if (lpszString)
+    {
+      int nLen;
+      LPCSTR p, r;
+      LPSTR q;
+
+      nLen = nSize >= 0 ? nSize : lstrlenA (lpszString);
+      p = lpszString;
+      r = p + nLen;
+      q = lpszResult = MemoryAlloc ((nLen * 6 + 3) * sizeof (CHAR));
+      *q++ = '\"';
+      while (p < r)
+        {
+          switch (*p)
+            {
+              case '\a': *q++ = '\\'; *q++ = 'a';  break;
+              case '\b': *q++ = '\\'; *q++ = 'b';  break;
+              case '\n': *q++ = '\\'; *q++ = 'n';  break;
+              case '\r': *q++ = '\\'; *q++ = 'r';  break;
+              case '\f': *q++ = '\\'; *q++ = 'f';  break;
+              case '\t': *q++ = '\\'; *q++ = 't';  break;
+              case '\v': *q++ = '\\'; *q++ = 'v';  break;
+              case '\\': *q++ = '\\'; *q++ = '\\'; break;
+              case '\?': *q++ = '\\'; *q++ = '\?'; break;
+              case '\'': *q++ = '\\'; *q++ = '\''; break;
+              case '\"': *q++ = '\\'; *q++ = '\"'; break;
+              default:
+                if (0x20 <= *p && *p <= 0x7e)
+                  {
+                    *q++ = *p;
+                  }
+                else
+                  {
+                    wsprintfA (q, "\\x%04X", (BYTE)*p);
+                    q += 6;
+                  }
+            }
+          p++;
+        }
+      *q++ = '\"';
+      *q = '\0';
+    }
+  else
+    {
+      lpszResult = StringDuplicateA ("NULL");
+    }
+  return lpszResult;
+}
+
+
+/*  文字列を正規化する
+    lpszString,文字列
+           RET,正規化された文字列                                           */
+LPWSTR WINAPI
+StringCanonicalizeW (LPCWSTR lpszString)
+{
+  return StringCanonicalizeExW (lpszString, -1);
+}
+
+
+/*  文字列を正規化する
+    lpszString,文字列
+           RET,正規化された文字列                                           */
+LPSTR WINAPI
+StringCanonicalizeA (LPCSTR lpszString)
+{
+  return StringCanonicalizeExA (lpszString, -1);
+}
+
+
+/*  文字列を分割する
+    lpszString,文字列
+    cDelimiter,文字
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI
+StringSplitW (LPCWSTR lpszString,
+              WCHAR   cDelimiter)
+{
+  LPWSTR *plpszArray = NULL;
+
+  if (lpszString)
+    {
+      int nCount = 2;
+      LPCWSTR p;
+      LPWSTR q, *r;
+
+      for (p = lpszString; *p != '\0'; p++)
+        if (*p == cDelimiter)
+          nCount++;
+      plpszArray = MemoryAlloc (nCount * sizeof (LPWSTR)
+                                    + (p - lpszString + 1) * sizeof (WCHAR));
+      q = (LPWSTR)(plpszArray + nCount);
+      lstrcpyW (q, lpszString);
+      r = plpszArray;
+      *r++ = q;
+      while (*q != '\0')
+        {
+          if (*q == cDelimiter)
+            {
+              *r++ = q + 1;
+              *q = '\0';
+            }
+          q++;
+        }
+    }
+  return plpszArray;
+}
+
+
+/*  文字列を分割する
+    lpszString,文字列
+    cDelimiter,文字
+           RET,分割された文字列                                             */
+LPSTR * WINAPI
+StringSplitA (LPCSTR lpszString,
+              CHAR   cDelimiter)
+{
+  LPSTR *plpszArray = NULL;
+
+  if (lpszString)
+    {
+      int nCount = 2;
+      LPCSTR p;
+      LPSTR q, *r;
+
+      for (p = lpszString; *p != '\0';
+                                    p += IsDBCSLeadByteEx (CP_ACP, *p) ? 2 : 1)
+        if (*p == cDelimiter)
+          nCount++;
+      plpszArray = MemoryAlloc (nCount * sizeof (LPSTR)
+                                    + (p - lpszString + 1) * sizeof (CHAR));
+      q = (LPSTR)(plpszArray + nCount);
+      lstrcpyA (q, lpszString);
+      r = plpszArray;
+      *r++ = q;
+      while (*q != '\0')
+        {
+          if (*q == cDelimiter)
+            {
+              *r++ = q + 1;
+              *q = '\0';
+            }
+          q += IsDBCSLeadByteEx (CP_ACP, *q) ? 2 : 1;
+        }
+    }
+  return plpszArray;
+}
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+       plpszArray,分割された文字列
+              RET,文字列                                                    */
+LPWSTR WINAPI
+StringJoinVW (LPCWSTR  lpszSeparator,
+              LPCWSTR *plpszArray)
+{
+  LPWSTR lpszResult = NULL;
+
+  if (plpszArray)
+    {
+      int i, nLen = 1;
+
+      for (i = 0; plpszArray[i]; i++)
+        nLen += lstrlenW (plpszArray[i]);
+      if (lpszSeparator && i > 1)
+        nLen += lstrlenW (lpszSeparator) * (i - 1);
+      if (nLen > 1)
+        {
+          lpszResult = MemoryAlloc (nLen * sizeof (WCHAR));
+          while (*plpszArray)
+            {
+              lstrcatW (lpszResult, *plpszArray);
+              plpszArray++;
+              if (lpszSeparator && *plpszArray)
+                lstrcatW (lpszResult, lpszSeparator);
+            }
+        }
+    }
+  return lpszResult;
+}
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+       plpszArray,分割された文字列
+              RET,文字列                                                    */
+LPSTR WINAPI
+StringJoinVA (LPCSTR  lpszSeparator,
+              LPCSTR *plpszArray)
+{
+  LPSTR lpszResult = NULL;
+
+  if (plpszArray)
+    {
+      int i, nLen = 1;
+
+      for (i = 0; plpszArray[i]; i++)
+        nLen += lstrlenA (plpszArray[i]);
+      if (lpszSeparator && i > 1)
+        nLen += lstrlenA (lpszSeparator) * (i - 1);
+      if (nLen > 1)
+        {
+          lpszResult = MemoryAlloc (nLen * sizeof (CHAR));
+          while (*plpszArray)
+            {
+              lstrcatA (lpszResult, *plpszArray);
+              plpszArray++;
+              if (lpszSeparator && *plpszArray)
+                lstrcatA (lpszResult, lpszSeparator);
+            }
+        }
+    }
+  return lpszResult;
+}
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+              RET,文字列                                                    */
+LPWSTR WINAPI
+StringJoinW (LPCWSTR lpszSeparator, ...)
+{
+  LPWSTR lpszResult = NULL;
+  LPCWSTR lpszString;
+  int i = 0, nLen = 1;
+  va_list arglist;
+
+  va_start (arglist, lpszSeparator);
+  while (lpszString = va_arg (arglist, LPCWSTR))
+    {
+      nLen += lstrlenW (lpszString);
+      i++;
+    }
+  va_end (arglist);
+  if (lpszSeparator && i > 1)
+    nLen += lstrlenW (lpszSeparator) * (i - 1);
+  if (nLen > 1)
+    {
+      lpszResult = MemoryAlloc (nLen * sizeof (WCHAR));
+      va_start (arglist, lpszSeparator);
+      lpszString = va_arg (arglist, LPCWSTR);
+      while (lpszString)
+        {
+          lstrcatW (lpszResult, lpszString);
+          lpszString = va_arg (arglist, LPCWSTR);
+          if (lpszSeparator && lpszString)
+            lstrcatW (lpszResult, lpszSeparator);
+        }
+      va_end (arglist);
+    }
+  return lpszResult;
+}
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+              RET,文字列                                                    */
+LPSTR WINAPI
+StringJoinA (LPCSTR lpszSeparator, ...)
+{
+  LPSTR lpszResult = NULL;
+  LPCSTR lpszString;
+  int i = 0, nLen = 1;
+  va_list arglist;
+
+  va_start (arglist, lpszSeparator);
+  while (lpszString = va_arg (arglist, LPCSTR))
+    {
+      nLen += lstrlenA (lpszString);
+      i++;
+    }
+  va_end (arglist);
+  if (lpszSeparator && i > 1)
+    nLen += lstrlenA (lpszSeparator) * (i - 1);
+  if (nLen > 1)
+    {
+      lpszResult = MemoryAlloc (nLen * sizeof (CHAR));
+      va_start (arglist, lpszSeparator);
+      lpszString = va_arg (arglist, LPCSTR);
+      while (lpszString)
+        {
+          lstrcatA (lpszResult, lpszString);
+          lpszString = va_arg (arglist, LPCSTR);
+          if (lpszSeparator && lpszString)
+            lstrcatA (lpszResult, lpszSeparator);
+        }
+      va_end (arglist);
+    }
+  return lpszResult;
+}
+
+
+/*  分割された文字列の数を求める
+    plpszArray,分割された文字列
+           RET,分割された文字列の数                                         */
+int WINAPI
+StringArrayLengthW (LPCWSTR *plpszArray)
+{
+  int nResult = 0;
+
+  if (plpszArray)
+    while (plpszArray[nResult])
+      nResult++;
+  return nResult;
+}
+
+
+/*  分割された文字列の数を求める
+    plpszArray,分割された文字列
+           RET,分割された文字列の数                                         */
+int WINAPI
+StringArrayLengthA (LPCSTR *plpszArray)
+{
+  int nResult = 0;
+
+  if (plpszArray)
+    while (plpszArray[nResult])
+      nResult++;
+  return nResult;
 }
 
 
@@ -420,7 +1077,8 @@ LoadFileA (LPCSTR  lpszFile,
             {
               DWORD dwRead;
 
-              lpBuffer = MemoryAlloc (dwSize + sizeof (CHAR));
+              lpBuffer = MemoryAlloc (((dwSize + 1) / sizeof (WCHAR) + 1)
+                                                            * sizeof (WCHAR));
               if (lpBuffer)
                 fResult = ReadFile (hFile, lpBuffer, dwSize, &dwRead, NULL)
                                                         && dwSize == dwRead;
@@ -750,7 +1408,7 @@ ListIndex (LPLIST lpStart,
 /*  線形リストの要素数を求める
     lpStart,線形リスト
         RET,要素数                                                          */
-INT WINAPI
+int WINAPI
 ListLength (LPLIST lpStart)
 {
   int nCount = 0;
@@ -1113,146 +1771,6 @@ HashEnum (LPHASH         lpHash,
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
-/*  文字列を正規化する
-    lpszString,文字列
-         nSize,文字数(負:NULL終端文字列)
-           RET,正規化された文字列                                           */
-LPWSTR WINAPI
-CanonicalStringExW (LPCWSTR lpszString,
-                    SSIZE_T nSize)
-{
-  LPWSTR lpszResult;
-
-  if (lpszString)
-    {
-      int nLen;
-      LPCWSTR p, r;
-      LPWSTR q;
-
-      nLen = nSize >= 0 ? nSize : lstrlenW (lpszString);
-      p = lpszString;
-      r = p + nLen;
-      q = lpszResult = MemoryAlloc ((nLen * 6 + 3) * sizeof (WCHAR));
-      *q++ = '\"';
-      while (p < r)
-        {
-          switch (*p)
-            {
-              case '\a': *q++ = '\\'; *q++ = 'a';  break;
-              case '\b': *q++ = '\\'; *q++ = 'b';  break;
-              case '\n': *q++ = '\\'; *q++ = 'n';  break;
-              case '\r': *q++ = '\\'; *q++ = 'r';  break;
-              case '\f': *q++ = '\\'; *q++ = 'f';  break;
-              case '\t': *q++ = '\\'; *q++ = 't';  break;
-              case '\v': *q++ = '\\'; *q++ = 'v';  break;
-              case '\\': *q++ = '\\'; *q++ = '\\'; break;
-              case '\?': *q++ = '\\'; *q++ = '\?'; break;
-              case '\'': *q++ = '\\'; *q++ = '\''; break;
-              case '\"': *q++ = '\\'; *q++ = '\"'; break;
-              default:
-                if (0x20 <= *p && *p <= 0x7e)
-                  {
-                    *q++ = *p;
-                  }
-                else
-                  {
-                    wsprintfW (q, L"\\x%04X", *p);
-                    q += 6;
-                  }
-            }
-          p++;
-        }
-      *q++ = '\"';
-      *q = '\0';
-    }
-  else
-    {
-      lpszResult = MemoryStringW (L"NULL");
-    }
-  return lpszResult;
-}
-
-
-/*  文字列を正規化する
-    lpszString,文字列
-         nSize,文字数(負:NULL終端文字列)
-           RET,正規化された文字列                                           */
-LPSTR WINAPI
-CanonicalStringExA (LPCSTR  lpszString,
-                    SSIZE_T nSize)
-{
-  LPSTR lpszResult;
-
-  if (lpszString)
-    {
-      int nLen;
-      LPCSTR p, r;
-      LPSTR q;
-
-      nLen = nSize >= 0 ? nSize : lstrlenA (lpszString);
-      p = lpszString;
-      r = p + nLen;
-      q = lpszResult = MemoryAlloc ((nLen * 6 + 3) * sizeof (CHAR));
-      *q++ = '\"';
-      while (p < r)
-        {
-          switch (*p)
-            {
-              case '\a': *q++ = '\\'; *q++ = 'a';  break;
-              case '\b': *q++ = '\\'; *q++ = 'b';  break;
-              case '\n': *q++ = '\\'; *q++ = 'n';  break;
-              case '\r': *q++ = '\\'; *q++ = 'r';  break;
-              case '\f': *q++ = '\\'; *q++ = 'f';  break;
-              case '\t': *q++ = '\\'; *q++ = 't';  break;
-              case '\v': *q++ = '\\'; *q++ = 'v';  break;
-              case '\\': *q++ = '\\'; *q++ = '\\'; break;
-              case '\?': *q++ = '\\'; *q++ = '\?'; break;
-              case '\'': *q++ = '\\'; *q++ = '\''; break;
-              case '\"': *q++ = '\\'; *q++ = '\"'; break;
-              default:
-                if (0x20 <= *p && *p <= 0x7e)
-                  {
-                    *q++ = *p;
-                  }
-                else
-                  {
-                    wsprintfA (q, "\\x%04X", (BYTE)*p);
-                    q += 6;
-                  }
-            }
-          p++;
-        }
-      *q++ = '\"';
-      *q = '\0';
-    }
-  else
-    {
-      lpszResult = MemoryStringA ("NULL");
-    }
-  return lpszResult;
-}
-
-
-/*  文字列を正規化する
-    lpszString,文字列
-           RET,正規化された文字列                                           */
-LPWSTR WINAPI
-CanonicalStringW (LPCWSTR lpszString)
-{
-  return CanonicalStringExW (lpszString, -1);
-}
-
-
-/*  文字列を正規化する
-    lpszString,文字列
-           RET,正規化された文字列                                           */
-LPSTR WINAPI
-CanonicalStringA (LPCSTR lpszString)
-{
-  return CanonicalStringExA (lpszString, -1);
-}
-
-
 /*  ja:コマンドラインの解析
     lpszCmdLine,コマンドライン文字列
             RET,引数へのポインタ,NULL:エラー                                */
@@ -1497,37 +2015,22 @@ MakeArgumentA (LPCSTR lpszCmdLine)
 
 /*  文字列をデバイスに出力する
     nStdHandle,デバイス
-      lpFormat,書式文字列                                                   */
+    lpszFormat,書式文字列                                                   */
 VOID WINAPI
 PrintConsoleW (DWORD   nStdHandle,
-               LPCWSTR lpFormat, ...)
+               LPCWSTR lpszFormat, ...)
 {
-  if (lpFormat)
+  if (lpszFormat)
     {
-      int n;
       va_list arglist;
       DWORD dwWrite;
-      LPSTR lpszMultiByte;
       LPWSTR lpszText = NULL;
-      SIZE_T uSize = 0;
 
-      va_start (arglist, lpFormat);
-      do
-        {
-          uSize += sizeof (WCHAR) * 4096;
-          lpszText = MemoryReAlloc (lpszText, uSize);
-          n  = wvnsprintfW (lpszText, uSize, lpFormat, arglist);
-        }
-      while (n < 0 || n >= uSize - 1);
+      va_start (arglist, lpszFormat);
+      wvasprintfW (&lpszText, lpszFormat, arglist);
       va_end (arglist);
-      uSize = WideCharToMultiByte (CP_ACP, 0, lpszText, -1,
-                                                        NULL, 0, NULL, NULL);
-      lpszMultiByte = MemoryAlloc (uSize);
-      WideCharToMultiByte (CP_ACP, 0, lpszText, -1,
-                                            lpszMultiByte, uSize, NULL, NULL);
-      WriteFile (GetStdHandle (nStdHandle), lpszMultiByte,
-                    lstrlenA (lpszMultiByte) * sizeof (CHAR), &dwWrite, NULL);
-      MemoryFree (lpszMultiByte);
+      WriteConsoleW (GetStdHandle (nStdHandle), lpszText,
+                                        lstrlenW (lpszText), &dwWrite, NULL);
       MemoryFree (lpszText);
     }
 }
@@ -1535,109 +2038,22 @@ PrintConsoleW (DWORD   nStdHandle,
 
 /*  文字列をデバイスに出力する
     nStdHandle,デバイス
-      lpFormat,書式文字列                                                   */
+    lpszFormat,書式文字列                                                   */
 VOID WINAPI
 PrintConsoleA (DWORD  nStdHandle,
-               LPCSTR lpFormat, ...)
+               LPCSTR lpszFormat, ...)
 {
-  if (lpFormat)
+  if (lpszFormat)
     {
-      int n;
       va_list arglist;
       DWORD dwWrite;
-      LPSTR lpszText = NULL;
-      SIZE_T uSize = 0;
+      LPSTR lpszText;
 
-      va_start (arglist, lpFormat);
-      do
-        {
-          uSize += sizeof (CHAR) * 4096;
-          lpszText = MemoryReAlloc (lpszText, uSize);
-          n  = wvnsprintfA (lpszText, uSize, lpFormat, arglist);
-        }
-      while (n < 0 || n >= uSize - 1);
+      va_start (arglist, lpszFormat);
+      wvasprintfA (&lpszText, lpszFormat, arglist);
       va_end (arglist);
-      WriteFile (GetStdHandle (nStdHandle), lpszText,
-                        lstrlenA (lpszText) * sizeof (CHAR), &dwWrite, NULL);
+      WriteConsoleA (GetStdHandle (nStdHandle), lpszText,
+                                        lstrlenA (lpszText), &dwWrite, NULL);
       MemoryFree (lpszText);
     }
 }
-
-
-/*  文字列を分割する
-    lpszString,文字列
-    cDelimiter,文字
-           RET,分割された文字列                                             */
-LPWSTR * WINAPI
-SplitStringW (LPCWSTR lpszString,
-              WCHAR   cDelimiter)
-{
-  LPWSTR *plpszArray = NULL;
-
-  if (lpszString)
-    {
-      int nCount = 2;
-      LPCWSTR p;
-      LPWSTR q, *r;
-
-      for (p = lpszString; *p != '\0'; p++)
-        if (*p == cDelimiter)
-          nCount++;
-      plpszArray = MemoryAlloc (nCount * sizeof (LPWSTR)
-                                    + (p - lpszString + 1) * sizeof (WCHAR));
-      q = (LPWSTR)(plpszArray + nCount);
-      lstrcpyW (q, lpszString);
-      r = plpszArray;
-      *r++ = q;
-      while (*q != '\0')
-        {
-          if (*q == cDelimiter)
-            {
-              *r++ = q + 1;
-              *q = '\0';
-            }
-          q++;
-        }
-    }
-  return plpszArray;
-}
-
-
-/*  文字列を分割する
-    lpszString,文字列
-    cDelimiter,文字
-           RET,分割された文字列                                             */
-LPSTR * WINAPI
-SplitStringA (LPCSTR lpszString,
-              CHAR   cDelimiter)
-{
-  LPSTR *plpszArray = NULL;
-
-  if (lpszString)
-    {
-      int nCount = 2;
-      LPCSTR p;
-      LPSTR q, *r;
-
-      for (p = lpszString; *p != '\0';
-                                    p += IsDBCSLeadByteEx (CP_ACP, *p) ? 2 : 1)
-        if (*p == cDelimiter)
-          nCount++;
-      plpszArray = MemoryAlloc (nCount * sizeof (LPSTR)
-                                    + (p - lpszString + 1) * sizeof (CHAR));
-      q = (LPSTR)(plpszArray + nCount);
-      lstrcpyA (q, lpszString);
-      r = plpszArray;
-      *r++ = q;
-      while (*q != '\0')
-        {
-          if (*q == cDelimiter)
-            {
-              *r++ = q + 1;
-              *q = '\0';
-            }
-          q += IsDBCSLeadByteEx (CP_ACP, *q) ? 2 : 1;
-        }
-    }
-  return plpszArray;
-}
