@@ -1,6 +1,6 @@
 /*
     common
-    copyright (c) 1998-2017 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2017 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -98,7 +98,7 @@ MemoryCopy (LPVOID  lpvDest,
     lpvBuf1,バッファ2
      uBytes,バイト数
         RET,0:等しい,負:バッファ1は小さい,正:バッファ1は大きい              */
-INT WINAPI
+int WINAPI
 MemoryCompare (LPCVOID lpvBuf1,
                LPCVOID lpvBuf2,
                SIZE_T  uBytes);
@@ -136,24 +136,255 @@ MemoryDuplicate (LPCVOID lpMemory,
                  SIZE_T  uBytes);
 
 
-/*  文字列を確保する
+#ifdef COMMON_NO_DEPRECATE
+# pragma deprecated(MemoryString, MemoryStringW, MemoryStringA)
+# define MemoryStringW StringDuplicateW
+# define MemoryStringA StringDuplicateA
+# ifdef UNICODE
+#  define MemoryString StringDuplicateW
+# else /* not UNICODE */
+#  define MemoryString StringDuplicateA
+# endif /* not UNICODE */
+#endif /* COMMON_NO_DEPRECATE */
+
+
+/******************************************************************************
+*                                                                             *
+******************************************************************************/
+/*  文字列を書式化する
+    ppszString,文字列
+    lpszFormat,書式文字列
+       arglist,引数
+           RET,文字数(終端のNULLを含まない)                                 */
+SSIZE_T WINAPI
+wvasprintfW (LPWSTR  *ppszString,
+             LPCWSTR  lpszFormat,
+             va_list  arglist);
+
+
+/*  文字列を書式化する
+    ppszString,文字列
+    lpszFormat,書式文字列
+       arglist,引数
+           RET,文字数(終端のNULLを含まない)                                 */
+SSIZE_T WINAPI
+wvasprintfA (LPSTR  *ppszString,
+             LPCSTR  lpszFormat,
+             va_list arglist);
+
+
+/*  文字列を書式化する
+    ppszString,文字列
+      lpFormat,書式文字列
+           RET,文字数(終端のNULLを含まない)                                 */
+SSIZE_T WINAPI
+wasprintfW (LPWSTR  *ppszString,
+            LPCWSTR  lpszFormat, ...);
+
+
+/*  文字列を書式化する
+    ppszString,文字列
+      lpFormat,書式文字列
+           RET,文字数(終端のNULLを含まない)                                 */
+SSIZE_T WINAPI
+wasprintfA (LPSTR  *ppszString,
+            LPCSTR  lpszFormat, ...);
+
+
+/*  文字列を複製する
     lpszText,文字列
-         RET,確保したメモリ,NULL:エラー                                     */
+       nSize,文字数(負:NULL終端文字列)
+         RET,複製したメモリ,NULL:エラー                                     */
 LPWSTR WINAPI
-MemoryStringW (LPCWSTR lpszText);
+StringDuplicateExW (LPCWSTR lpszText,
+                    SSIZE_T nSize);
 
 
-/*  文字列を確保する
+/*  文字列を複製する
     lpszText,文字列
-         RET,確保したメモリ,NULL:エラー                                     */
+       nSize,文字数(負:NULL終端文字列)
+         RET,複製したメモリ,NULL:エラー                                     */
 LPSTR WINAPI
-MemoryStringA (LPCSTR lpszText);
+StringDuplicateExA (LPCSTR  lpszText,
+                    SSIZE_T nSize);
+
+
+/*  文字列を複製する
+    lpszText,文字列
+         RET,複製したメモリ,NULL:エラー                                     */
+LPWSTR WINAPI
+StringDuplicateW (LPCWSTR lpszText);
+
+
+/*  文字列を複製する
+    lpszText,文字列
+         RET,複製したメモリ,NULL:エラー                                     */
+LPSTR WINAPI
+StringDuplicateA (LPCSTR lpszText);
+
+
+/*  マルチバイト文字列をワイド文字列に変換する
+    lpszMultiByte,マルチバイト文字列
+     cchMultiByte,文字数(負:NULL終端文字列)
+              RET,ワイド文字列                                              */
+LPWSTR WINAPI
+StringMultiByteToWideCharEx (LPCSTR  lpszMultiByte,
+                             SSIZE_T cchMultiByte);
+
+
+/*  ワイド文字列をマルチバイト文字列に変換する
+    lpszWideChar,ワイド文字列
+     cchWideChar,文字数(負:NULL終端文字列)
+             RET,マルチバイト文字列                                         */
+LPSTR WINAPI
+StringWideCharToMultiByteEx (LPCWSTR lpszWideChar,
+                             SSIZE_T cchWideChar);
+
+
+/*  マルチバイト文字列をワイド文字列に変換する
+    lpszMultiByte,マルチバイト文字列
+              RET,ワイド文字列                                              */
+LPWSTR WINAPI
+StringMultiByteToWideChar (LPCSTR lpszMultiByte);
+
+
+/*  ワイド文字列をマルチバイト文字列に変換する
+    lpszWideChar,ワイド文字列
+             RET,マルチバイト文字列                                         */
+LPSTR WINAPI
+StringWideCharToMultiByte (LPCWSTR lpszWideChar);
+
+
+/*  文字列を正規化する
+    lpszString,文字列
+         nSize,文字数(負:NULL終端文字列)
+           RET,正規化された文字列                                           */
+LPWSTR WINAPI
+StringCanonicalizeExW (LPCWSTR lpszString,
+                       SSIZE_T nSize);
+
+
+/*  文字列を正規化する
+    lpszString,文字列
+         nSize,文字数(負:NULL終端文字列)
+           RET,正規化された文字列                                           */
+LPSTR WINAPI
+StringCanonicalizeExA (LPCSTR  lpszString,
+                       SSIZE_T nSize);
+
+
+/*  文字列を正規化する
+    lpszString,文字列
+           RET,正規化された文字列                                           */
+LPWSTR WINAPI
+StringCanonicalizeW (LPCWSTR lpszString);
+
+
+/*  文字列を正規化する
+    lpszString,文字列
+           RET,正規化された文字列                                           */
+LPSTR WINAPI
+StringCanonicalizeA (LPCSTR lpszString);
+
+
+/*  文字列を分割する
+    lpszString,文字列
+    cDelimiter,文字
+           RET,分割された文字列                                             */
+LPWSTR * WINAPI
+StringSplitW (LPCWSTR lpszString,
+              WCHAR   cDelimiter);
+
+
+/*  文字列を分割する
+    lpszString,文字列
+    cDelimiter,文字
+           RET,分割された文字列                                             */
+LPSTR * WINAPI
+StringSplitA (LPCSTR lpszString,
+              CHAR   cDelimiter);
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+       plpszArray,分割された文字列
+              RET,文字列                                                    */
+LPWSTR WINAPI
+StringJoinVW (LPCWSTR  lpszSeparator,
+              LPCWSTR *plpszArray);
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+       plpszArray,分割された文字列
+              RET,文字列                                                    */
+LPSTR WINAPI
+StringJoinVA (LPCSTR  lpszSeparator,
+              LPCSTR *plpszArray);
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+              RET,文字列                                                    */
+LPWSTR WINAPI
+StringJoinW (LPCWSTR lpszSeparator, ...);
+
+
+/*  文字列を結合する
+    lpszSeparator,区切り文字列
+              RET,文字列                                                    */
+LPSTR WINAPI
+StringJoinA (LPCSTR lpszSeparator, ...);
+
+
+/*  分割された文字列の数を求める
+    plpszArray,分割された文字列
+           RET,分割された文字列の数                                         */
+int WINAPI
+StringArrayLengthW (LPCWSTR *plpszArray);
+
+
+/*  分割された文字列の数を求める
+    plpszArray,分割された文字列
+           RET,分割された文字列の数                                         */
+int WINAPI
+StringArrayLengthA (LPCSTR *plpszArray);
 
 
 #ifdef UNICODE
-# define MemoryString MemoryStringW
+# define wvasprintf wvasprintfW
+# define wasprintf wasprintfW
+# define StringDuplicateEx StringDuplicateExW
+# define StringDuplicate StringDuplicateW
+# define StringFromMultiByteEx StringMultiByteToWideCharEx
+# define StringFromMultiByte StringMultiByteToWideChar
+# define StringFromWideCharEx StringDuplicateExW
+# define StringFromWideChar StringDuplicateW
+# define StringToMultiByteEx StringWideCharToMultiByteEx
+# define StringToMultiByte StringWideCharToMultiByte
+# define StringToWideCharEx StringDuplicateExW
+# define StringToWideChar StringDuplicateW
+# define StringCanonicalizeEx StringCanonicalizeExW
+# define StringCanonicalize StringCanonicalizeW
+# define StringSplit StringSplitW
+# define StringArrayLength StringArrayLengthW
 #else /* not UNICODE */
-# define MemoryString MemoryStringA
+# define wvasprintf wvasprintfA
+# define wasprintf wasprintfA
+# define StringDuplicateEx StringDuplicateExA
+# define StringDuplicate StringDuplicateA
+# define StringFromMultiByteEx StringDuplicateExA
+# define StringFromMultiByte StringDuplicateA
+# define StringFromWideCharEx StringWideCharToMultiByteEx
+# define StringFromWideChar StringWideCharToMultiByte
+# define StringToMultiByteEx StringDuplicateExA
+# define StringToMultiByte StringDuplicateA
+# define StringToWideCharEx StringMultiByteToWideCharEx
+# define StringToWideChar StringMultiByteToWideChar
+# define StringCanonicalizeEx StringCanonicalizeExA
+# define StringCanonicalize StringCanonicalizeA
+# define StringSplit StringSplitA
+# define StringArrayLength StringArrayLengthA
 #endif /* not UNICODE */
 
 
@@ -328,7 +559,7 @@ ListIndex (LPLIST lpStart,
 /*  線形リストの要素数を求める
     lpStart,線形リスト
         RET,要素数                                                          */
-INT WINAPI
+int WINAPI
 ListLength (LPLIST lpStart);
 
 
@@ -511,38 +742,6 @@ HashEnum (LPHASH         lpHash,
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
-/*  文字列を正規化する
-    lpszString,文字列
-         nSize,文字数(負:NULL終端文字列)
-           RET,正規化された文字列                                           */
-LPWSTR WINAPI
-CanonicalStringExW (LPCWSTR lpszString,
-                    SSIZE_T nSize);
-
-
-/*  文字列を正規化する
-    lpszString,文字列
-         nSize,文字数(負:NULL終端文字列)
-           RET,正規化された文字列                                           */
-LPSTR WINAPI
-CanonicalStringExA (LPCSTR  lpszString,
-                    SSIZE_T nSize);
-
-
-/*  文字列を正規化する
-    lpszString,文字列
-           RET,正規化された文字列                                           */
-LPWSTR WINAPI
-CanonicalStringW (LPCWSTR lpszString);
-
-
-/*  文字列を正規化する
-    lpszString,文字列
-           RET,正規化された文字列                                           */
-LPSTR WINAPI
-CanonicalStringA (LPCSTR lpszString);
-
-
 /*  ja:コマンドラインの解析
     lpszCmdLine,コマンドライン文字列
             RET,引数へのポインタ,NULL:エラー                                */
@@ -559,53 +758,51 @@ MakeArgumentA (LPCSTR lpszCmdLine);
 
 /*  文字列をデバイスに出力する
     nStdHandle,デバイス
-      lpFormat,書式文字列                                                   */
+    lpszFormat,書式文字列                                                   */
 VOID WINAPI
 PrintConsoleW (DWORD   nStdHandle,
-               LPCWSTR lpFormat, ...);
+               LPCWSTR lpszFormat, ...);
 
 
 /*  文字列をデバイスに出力する
     nStdHandle,デバイス
-      lpFormat,書式文字列                                                   */
+    lpszFormat,書式文字列                                                   */
 VOID WINAPI
 PrintConsoleA (DWORD  nStdHandle,
-               LPCSTR lpFormat, ...);
-
-
-/*  文字列を分割する
-    lpszString,文字列
-    cDelimiter,文字
-           RET,分割された文字列                                             */
-LPWSTR * WINAPI
-SplitStringW (LPCWSTR lpszString,
-              WCHAR   cDelimiter);
-
-
-/*  文字列を分割する
-    lpszString,文字列
-    cDelimiter,文字
-           RET,分割された文字列                                             */
-LPSTR * WINAPI
-SplitStringA (LPCSTR lpszString,
-              CHAR   cDelimiter);
+               LPCSTR lpszFormat, ...);
 
 
 #ifdef UNICODE
-# define CanonicalStringEx CanonicalStringExW
-# define CanonicalString CanonicalStringW
 # define MakeArgument MakeArgumentW
 # define PrintConsole PrintConsoleW
-# define SplitString SplitStringW
 #else /* not UNICODE */
-# define CanonicalStringEx CanonicalStringExA
-# define CanonicalString CanonicalStringA
 # define MakeArgument MakeArgumentA
 # define PrintConsole PrintConsoleA
-# define SplitString SplitStringA
 #endif /* not UNICODE */
 
 
+#ifdef COMMON_NO_DEPRECATE
+# pragma deprecated(CanonicalStringEx, CanonicalStringExW, CanonicalStringExA)
+# pragma deprecated(CanonicalString, CanonicalStringW, CanonicalStringA)
+# pragma deprecated(SplitString, SplitStringW, SplitStringA)
+# define CanonicalStringExW StringCanonicalExW
+# define CanonicalStringW StringCanonicalW
+# define SplitStringW StringSplitW
+# define CanonicalStringExA StringCanonicalExA
+# define CanonicalStringA StringCanonicalA
+# define SplitStringA StringSplitA
+# ifdef UNICODE
+#  define CanonicalStringEx StringCanonicalExW
+#  define CanonicalString StringCanonicalW
+#  define SplitString StringSplitW
+# else /* not UNICODE */
+#  define CanonicalStringEx StringCanonicalExA
+#  define CanonicalString StringCanonicalA
+#  define SplitString StringSplitA
+# endif /* not UNICODE */
+#endif /* COMMON_NO_DEPRECATE */
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
