@@ -1,6 +1,6 @@
 /*
     loaderex
-    copyright (c) 1998-2017 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2017 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,29 +16,68 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include <windows.h>
-#include <tchar.h>
 #include <shlwapi.h>
+#include <tchar.h>
 #include <stdio.h>
 #include "common.h"
 
 
 /*  ヘルプ表示
-          fp,ストリーム,
-    lpszFile,ファイル名                                                     */
-VOID WINAPI
-help (FILE *fp,
-      LPCTSTR lpszFile)
+    nStdHandle,デバイス
+      lpszFile,ファイル名                                                   */
+static VOID WINAPI
+PrintHelp (DWORD   nStdHandle,
+           LPCTSTR lpszFile)
 {
-  _ftprintf (fp,
-_T("loaderex "VERSION" ("BUILD_ENVIRONMENT")\n")
-_T("copyright (c) 1998-2017 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org\n")
+  LPTSTR lpszInternalName = NULL;
+  LPTSTR lpszProductVersion = NULL;
+  LPTSTR lpszLegalCopyright = NULL;
+  TCHAR szFile[MAX_PATH];
+
+  if (GetModuleFileName (NULL, szFile, MAX_PATH) > 0)
+    {
+      DWORD dwData, dwHandle;
+
+      dwData = GetFileVersionInfoSize (szFile, &dwHandle);
+      if(dwData > 0)
+        {
+          LPVOID lpData;
+
+          lpData = MemoryAlloc (dwData);
+          if (GetFileVersionInfo (szFile, dwHandle, dwData, lpData))
+            {
+              LPVOID lpVer;
+              UINT uVer;
+
+              if (VerQueryValue (lpData,
+                            _T("\\StringFileInfo\\040904e4\\InternalName"),
+                                                                &lpVer, &uVer))
+                lpszInternalName = StringDuplicateEx (lpVer, uVer);
+              if (VerQueryValue (lpData,
+                            _T("\\StringFileInfo\\040904e4\\ProductVersion"),
+                                                                &lpVer, &uVer))
+                lpszProductVersion = StringDuplicateEx (lpVer, uVer);
+              if (VerQueryValue (lpData,
+                            _T("\\StringFileInfo\\040904e4\\LegalCopyright"),
+                                                                &lpVer, &uVer))
+                lpszLegalCopyright = StringDuplicateEx (lpVer, uVer);
+            }
+          MemoryFree (lpData);
+        }
+    }
+  PrintConsole (nStdHandle,
+_T("%s %s ("BUILD_ENVIRONMENT")\n")
+_T("%s\n")
 _T("\n")
 _T("Usage: %s [option...] file\n")
 _T("\n")
 _T("  -a, --address=HEX      address\n")
 _T("  -e, --entry=HEX        entry point\n")
 _T("  -f, --file=FILE        open file\n")
-_T("\n"), lpszFile);
+_T("\n"), lpszInternalName, lpszProductVersion, lpszLegalCopyright, lpszFile);
+  MemoryFree (lpszInternalName);
+  MemoryFree (lpszProductVersion);
+  MemoryFree (lpszLegalCopyright);
 }
 
 
@@ -48,16 +87,17 @@ _tmain (int     argc,
 {
   int i = 1, ret = -1;
   LPVOID lpAddress = NULL, lpCode = NULL, lpEntry;
-  DWORD dwAddress = 0, dwEntry = 0, dwCode = 0, dwPadding = 0;
+  DWORD dwAddress = 0, dwEntry = 0, dwCode = 0, dwPadding = 0, dwSize;
   HANDLE hFile = INVALID_HANDLE_VALUE;
   LPCTSTR lpszFile = NULL;
 
   while (i < argc)
     {
-      if (StrCmpI (argv[i], _T("-h")) == 0
+      if (StrCmpI (argv[i], _T("/?")) == 0 || StrCmpI (argv[i], _T("-?")) == 0
+       || StrCmpI (argv[i], _T("/h")) == 0 || StrCmpI (argv[i], _T("-h")) == 0
        || StrCmpI (argv[i], _T("--help")) == 0)
         {
-          help (stdout, argv[0]);
+          PrintHelp (STD_OUTPUT_HANDLE, argv[0]);
           ret = 0;
           goto final;
         }
@@ -93,20 +133,21 @@ _tmain (int     argc,
           lpCode = LoadFile (argv[i], &dwCode);
           if (!lpCode)
             {
-              _ftprintf (stderr, _T("Error : LoadFile(\"%s\")\n"), argv[i]);
+              PrintConsole (STD_ERROR_HANDLE,
+                                    _T("Error : LoadFile(\"%s\")\n"), argv[i]);
               goto final;
             }
           i++;
         }
       else
         {
-          help (stderr, argv[0]);
+          PrintHelp (STD_ERROR_HANDLE, argv[0]);
           goto final;
         }
     }
   if (!lpCode || dwAddress == (DWORD)-1 || dwEntry == (DWORD)-1)
     {
-      help (stderr, argv[0]);
+      PrintHelp (STD_ERROR_HANDLE, argv[0]);
       goto final;
     }
 
@@ -119,12 +160,16 @@ _tmain (int     argc,
                 - dwAddress / SystemInfo.dwPageSize * SystemInfo.dwPageSize;
       dwEntry += dwPadding;
     }
-  lpAddress = VirtualAlloc ((LPVOID)(dwAddress - dwPadding),
-                    dwCode + dwPadding, MEM_COMMIT, PAGE_EXECUTE_READWRITE);
+  dwSize = dwCode + dwPadding;
+  lpAddress = VirtualAlloc ((LPVOID)(dwAddress - dwPadding), dwSize,
+                                        MEM_RESERVE, PAGE_EXECUTE_READWRITE);
+  if (lpAddress)
+    lpAddress = VirtualAlloc (lpAddress, dwSize,
+                                        MEM_COMMIT, PAGE_EXECUTE_READWRITE);
   if (!lpAddress)
     {
-      _ftprintf (stderr, _T("Error : VirtualAlloc (%08X, %08X)\n"),
-                                                            dwAddress, dwCode);
+      PrintConsole (STD_ERROR_HANDLE,
+                _T("Error : VirtualAlloc (%08X, %08X)\n"), dwAddress, dwSize);
       goto final;
     }
   MemoryCopy ((LPBYTE)lpAddress + dwPadding, lpCode, dwCode);
@@ -134,12 +179,13 @@ _tmain (int     argc,
                                 OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
       if (hFile == INVALID_HANDLE_VALUE)
         {
-          _ftprintf (stderr, _T("Error : CreateFile (\"%s\")\n"), lpszFile);
+          PrintConsole (STD_ERROR_HANDLE,
+                                _T("Error : CreateFile (\"%s\")\n"), lpszFile);
           goto final;
         }
     }
   lpEntry = (LPBYTE)lpAddress + dwEntry;
-  _tprintf (_T("%08X\n"), (DWORD)lpEntry);
+  PrintConsole (STD_OUTPUT_HANDLE, _T("%08X %08X\n"), (DWORD)lpAddress);
   __asm int 3
   ((VOID (WINAPI *)(VOID))(lpEntry)) ();
   if (hFile != INVALID_HANDLE_VALUE)
