@@ -1,6 +1,6 @@
 /*
     mki
-    copyright (c) 2000-2017 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 2000-2017 Kazuki Iwamoto https://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,13 +16,68 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include <windows.h>
-#include <tlhelp32.h>
+#include <shlwapi.h>
 #include <tchar.h>
 #include "common.h"
 #include "peimage.h"
 #include "snap.h"
 
 
+/*  ヘルプ表示
+    nStdHandle,デバイス
+      lpszFile,ファイル名                                                   */
+static VOID WINAPI
+PrintHelp (DWORD   nStdHandle,
+           LPCTSTR lpszFile)
+{
+  LPTSTR lpszInternalName = NULL;
+  LPTSTR lpszProductVersion = NULL;
+  LPTSTR lpszLegalCopyright = NULL;
+  TCHAR szFile[MAX_PATH];
+
+  if (GetModuleFileName (NULL, szFile, MAX_PATH) > 0)
+    {
+      DWORD dwData, dwHandle;
+
+      dwData = GetFileVersionInfoSize (szFile, &dwHandle);
+      if(dwData > 0)
+        {
+          LPVOID lpData;
+
+          lpData = MemoryAlloc (dwData);
+          if (GetFileVersionInfo (szFile, dwHandle, dwData, lpData))
+            {
+              LPVOID lpVer;
+              UINT uVer;
+
+              if (VerQueryValue (lpData,
+                            _T("\\StringFileInfo\\040904e4\\InternalName"),
+                                                                &lpVer, &uVer))
+                lpszInternalName = StringDuplicateEx (lpVer, uVer);
+              if (VerQueryValue (lpData,
+                            _T("\\StringFileInfo\\040904e4\\ProductVersion"),
+                                                                &lpVer, &uVer))
+                lpszProductVersion = StringDuplicateEx (lpVer, uVer);
+              if (VerQueryValue (lpData,
+                            _T("\\StringFileInfo\\040904e4\\LegalCopyright"),
+                                                                &lpVer, &uVer))
+                lpszLegalCopyright = StringDuplicateEx (lpVer, uVer);
+            }
+          MemoryFree (lpData);
+        }
+    }
+  PrintConsole (nStdHandle,
+_T("%s %s ("BUILD_ENVIRONMENT")\n")
+_T("%s\n")
+_T("\n")
+_T("Usage: %s dmpfile impfile zzzfile\n")
+_T("\n"), lpszInternalName, lpszProductVersion, lpszLegalCopyright, lpszFile);
+  MemoryFree (lpszInternalName);
+  MemoryFree (lpszProductVersion);
+  MemoryFree (lpszLegalCopyright);
+}
+
+
 int
 _tmain (int     argc,
         _TCHAR *argv[])
@@ -41,14 +96,17 @@ _tmain (int     argc,
   PIMAGE_IMPORT_DESCRIPTOR piid;
   PDLL_LIST dl = NULL;
 
+  for (i = 1; i < argc; i++)
+    if (StrCmpI (argv[i], _T("/?")) == 0 || StrCmpI (argv[i], _T("-?")) == 0
+     || StrCmpI (argv[i], _T("/h")) == 0 || StrCmpI (argv[i], _T("-h")) == 0
+     || StrCmpI (argv[i], _T("--help")) == 0)
+      {
+        PrintHelp (STD_OUTPUT_HANDLE, argv[0]);
+        return 0;
+      }
   if (argc != 4)
     {
-      PrintConsole (STD_ERROR_HANDLE,
-_T("mki "VERSION" ("BUILD_ENVIRONMENT")\n")
-_T("copyright (c) 2000-2017 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org\n")
-_T("\n")
-_T("Usage: %s dmpfile impfile zzzfile\n")
-_T("\n"), argv[0]);
+      PrintHelp (STD_ERROR_HANDLE, argv[0]);
       return -1;
     }
 
@@ -91,13 +149,13 @@ _T("\n"), argv[0]);
     else if (lpszList[i + 1] != '\n')
       lpszList[j++] = '\n';
   lpszList[j] = '\0';
-  lpszLine = SplitStringA (lpszList, '\n');
+  lpszLine = StringSplitA (lpszList, '\n');
   MemoryFree (lpszList);
   for (i = 0; lpszLine[i]; i++)
     {
       LPSTR *lpszText;
 
-      lpszText = SplitStringA (lpszLine[i], ',');
+      lpszText = StringSplitA (lpszLine[i], ',');
       if (lpszText)
         {
           if (lpszText[0] && lpszText[1] && lpszText[2]
@@ -107,17 +165,17 @@ _T("\n"), argv[0]);
 
               dwAddress = HexA (lpszText[0]);
               if (nDll <= 0 || dwPrev + sizeof (DWORD) != dwAddress
-                        || lstrcmpiA (dl[nDll - 1].szModule, lpszText[2]) != 0)
+                    || lstrcmpiA (dl[nDll - 1].lpszModule, lpszText[2]) != 0)
                 {
                   nDll++;
                   dl = MemoryReAlloc (dl, nDll * sizeof (DLL_LIST));
-                  lstrcpyA (dl[nDll - 1].szModule, lpszText[2]);
+                  dl[nDll - 1].lpszModule = StringDuplicateA (lpszText[2]);
                 }
               dl[nDll - 1].al = MemoryReAlloc (dl[nDll - 1].al,
                                 (dl[nDll - 1].nApi + 1) * sizeof (API_LIST));
               dl[nDll - 1].al[dl[nDll - 1].nApi].dwAddress = dwAddress;
               dl[nDll - 1].al[dl[nDll - 1].nApi].lpszName
-                                                = MemoryStringA (lpszText[3]);
+                                            = StringDuplicateA (lpszText[3]);
               dl[nDll - 1].nApi++;
               dwPrev = dwAddress;
             }
@@ -129,7 +187,7 @@ _T("\n"), argv[0]);
   for (i = 0; i < nDll; i++)
     {
       dwImport += (dl[i].nApi + 1) * sizeof (DWORD)
-                                        + (lstrlenA (dl[i].szModule) + 2 & ~1);
+                                    + (lstrlenA (dl[i].lpszModule) + 2 & ~1);
       for (j = 0; j < dl[i].nApi; j++)
         dwImport += 2 + (lstrlenA (dl[i].al[j].lpszName) + 2 & ~1);
     }
@@ -148,8 +206,8 @@ _T("\n"), argv[0]);
   for (i = 0; i < nDll; i++)
     {
       piid[i].Name = lpszName - lpbImage;
-      lstrcpyA (lpszName, dl[i].szModule);
-      lpszName += lstrlenA (dl[i].szModule) + 2 & ~1;
+      lstrcpyA (lpszName, dl[i].lpszModule);
+      lpszName += lstrlenA (dl[i].lpszModule) + 2 & ~1;
       piid[i].OriginalFirstThunk = (LPBYTE)lpdwTable - lpbImage;
       piid[i].FirstThunk = dl[i].al[0].dwAddress;
       for (j = 0; j < dl[i].nApi; j++)
@@ -193,6 +251,7 @@ _T("\n"), argv[0]);
             dwMax = dl[i].al[j].dwAddress;
           MemoryFree (dl[i].al[j].lpszName);
         }
+      MemoryFree (dl[i].lpszModule);
       MemoryFree (dl[i].al);
     }
   MemoryFree (dl);
